﻿using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 비트맵 헬퍼
    /// </summary>
    public static class BitmapHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 비트맵 복사하기 - CopyBitmap(sourceBitmap, targetWidth)

        /// <summary>
        /// 비트맵 복사하기
        /// </summary>
        /// <param name="sourceBitmap">소스 비트맵</param>
        /// <param name="targetWidth">타겟 너비</param>
        /// <returns>비트맵</returns>
        public static Bitmap CopyBitmap(Bitmap sourceBitmap, int targetWidth)
        {
            float ratio = 1.0f;

            int maximumSize = sourceBitmap.Width > sourceBitmap.Height ? sourceBitmap.Width : sourceBitmap.Height;

            ratio = (float)maximumSize / (float)targetWidth;

            Bitmap targetBitmap = (sourceBitmap.Width > sourceBitmap.Height ? new Bitmap(targetWidth, (int)(sourceBitmap.Height / ratio)) :
                                                                              new Bitmap((int)(sourceBitmap.Width / ratio), targetWidth));

            using(Graphics targetGraphics = Graphics.FromImage(targetBitmap))
            {
                targetGraphics.CompositingQuality = CompositingQuality.HighQuality;
                targetGraphics.InterpolationMode  = InterpolationMode.HighQualityBicubic;
                targetGraphics.PixelOffsetMode    = PixelOffsetMode.HighQuality;

                targetGraphics.DrawImage
                (
                    sourceBitmap,
                    new Rectangle(0, 0, targetBitmap.Width, targetBitmap.Height),
                    new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height),
                    GraphicsUnit.Pixel
                );

                targetGraphics.Flush();
            }

            return targetBitmap;
        }

        #endregion
        #region 비트맵 블렌딩하기 - BelndBitmap(sourceBitmap1, sourceBitmap2, blueBlendType, greenBlendType, redBlendType)

        /// <summary>
        /// 비트맵 블렌딩하기
        /// </summary>
        /// <param name="sourceBitmap1">소스 비트맵 1</param>
        /// <param name="sourceBitmap2">소스 비트맵 2</param>
        /// <param name="blueBlendType">청색 블렌딩 타입</param>
        /// <param name="greenBlendType">녹색 블렌딩 타입</param>
        /// <param name="redBlendType">적색 블렌딩 타입</param>
        /// <returns>비트맵</returns>
        public static Bitmap BelndBitmap
        (
            Bitmap           sourceBitmap1,
            Bitmap           sourceBitmap2,
            BitwiseBlendType blueBlendType,
            BitwiseBlendType greenBlendType,
            BitwiseBlendType redBlendType
        )
        {
            BitmapData sourceBitmapData1 = sourceBitmap1.LockBits
            (
                new Rectangle(0, 0, sourceBitmap1.Width, sourceBitmap1.Height),
                ImageLockMode.ReadOnly,
                PixelFormat.Format32bppArgb
            );

            byte[] sourceByteArray1 = new byte[sourceBitmapData1.Stride * sourceBitmapData1.Height];

            Marshal.Copy(sourceBitmapData1.Scan0, sourceByteArray1, 0, sourceByteArray1.Length);

            sourceBitmap1.UnlockBits(sourceBitmapData1);

            BitmapData sourceBitmapData2 = sourceBitmap2.LockBits
            (
                new Rectangle(0, 0, sourceBitmap2.Width, sourceBitmap2.Height),
                ImageLockMode.ReadOnly,
                PixelFormat.Format32bppArgb
            );

            byte[] sourceByteArray2 = new byte[sourceBitmapData2.Stride * sourceBitmapData2.Height];

            Marshal.Copy(sourceBitmapData2.Scan0, sourceByteArray2, 0, sourceByteArray2.Length);

            sourceBitmap2.UnlockBits(sourceBitmapData2);

            int blue  = 0;
            int green = 0;
            int red   = 0;

            for(int k = 0; (k + 4 < sourceByteArray1.Length) && (k + 4 < sourceByteArray2.Length); k += 4)
            {
                if(blueBlendType == BitwiseBlendType.AND)
                {
                    blue = sourceByteArray1[k] & sourceByteArray2[k];
                }
                else if(blueBlendType == BitwiseBlendType.OR)
                {
                    blue = sourceByteArray1[k] | sourceByteArray2[k];
                }
                else if(blueBlendType == BitwiseBlendType.XOR)
                {
                    blue = sourceByteArray1[k] ^ sourceByteArray2[k];
                }

                if(greenBlendType == BitwiseBlendType.AND)
                {
                    green = sourceByteArray1[k+1] & sourceByteArray2[k+1];
                }
                else if(greenBlendType == BitwiseBlendType.OR)
                {
                    green = sourceByteArray1[k+1] | sourceByteArray2[k+1];
                }
                else if(greenBlendType == BitwiseBlendType.XOR)
                {
                    green = sourceByteArray1[k+1] ^ sourceByteArray2[k+1];
                }

                if(redBlendType == BitwiseBlendType.AND)
                {
                    red = sourceByteArray1[k+2] & sourceByteArray2[k+2];
                }
                else if(redBlendType == BitwiseBlendType.OR)
                {
                    red = sourceByteArray1[k+2] | sourceByteArray2[k+2];
                }
                else if(redBlendType == BitwiseBlendType.XOR)
                {
                    red = sourceByteArray1[k+2] ^ sourceByteArray2[k+2];
                }

                if(blue < 0)
                {
                    blue = 0;
                }
                else if(blue > 255)
                {
                    blue = 255;
                }

                if(green < 0)
                {
                    green = 0;
                }
                else if(green > 255)
                {
                    green = 255;
                }

                if(red < 0)
                {
                    red = 0;
                }
                else if(red > 255)
                {
                    red = 255;
                }

                sourceByteArray1[k    ] = (byte)blue;
                sourceByteArray1[k + 1] = (byte)green;
                sourceByteArray1[k + 2] = (byte)red;
            }

            Bitmap targetBitmap = new Bitmap(sourceBitmap1.Width, sourceBitmap1.Height);

            BitmapData targetBitmapData = targetBitmap.LockBits
            (
                new Rectangle(0, 0, targetBitmap.Width, targetBitmap.Height),
                ImageLockMode.WriteOnly,
                PixelFormat.Format32bppArgb
            );

            Marshal.Copy(sourceByteArray1, 0, targetBitmapData.Scan0, sourceByteArray1.Length);

            targetBitmap.UnlockBits(targetBitmapData);

            return targetBitmap;
        }

        #endregion
    }
}